<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento;

require_once("UniModul.php");
require_once("UniAdapterModel.php");

use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItem;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItemType;
use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\OrderStatus;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;
use Platiti\GPWebPayGpeNs\UniModul\Http;

use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;

use Mage;
use Exception;

class UniAdapterController extends \Mage_Core_Controller_Front_Action
{
	/**
	 * @var string
	 */
	protected $uniModulName = null; //predefinuje binder
	/**
	 * @var UniModul
	 */
	protected $uniModul = null;
	/**
	 * @var string
	 */
	protected $subMethod = '';

	/**
	 * @return void
	 */
	protected function initUniModul() {
		if ($this->uniModul == null) {
			$uniFact = new UniModulFactory();
			$configSetting = $this->getConfigDataUni($this->uniModulName, $this->subMethod);
			$configSetting->uniModulConfig->funcProcessReplyStatus = function($orderReplyStatus) {$this->funcProcessReplyStatus($orderReplyStatus);};
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
		}
	}

	/**
	 * @param string $uniModulName
	 * @param string $subMethod
	 *
	 * @return ConfigSetting
	 */
	protected function getConfigDataUni($uniModulName, $subMethod) {
		$model = new UniAdapterModel();
		return $model->getConfigDataUniExternal($uniModulName, $subMethod);
	}

	/**
	 * @return void
	 */
	public function redirectAction() {
		if (is_null($this->uniModulName)) return;  // @phpstan-ignore-line better than checking if is null everywhere
		UniErr::begin();
		$orderIncrementId = $this->getRequest()->getParam('orderId');
		$subMethod = '';
		if ($orderIncrementId === null) {
			/** @var \PhpStan_Mage_Checkout_Model_Session */
			$checkout = checkAndGetSingleton('checkout/session');
			$orderIncrementId = $checkout->getLastRealOrderId();
			$subMethod = $checkout->getUniAdapterSubmethod(); // magic (from model) - implemented in Varien_Object->__call
		}

		/** @var \PhpStan_Mage_Sales_Model_Order */
		$orderModel = checkAndGetModel('sales/order');
		$order = $orderModel->loadByIncrementId($orderIncrementId);
		$orderToPayInfo = $this->getOrderToPayInfo($order, $subMethod);
		$this->initUniModul();

		$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

		if (!$order->getEmailSent()) {
			$order->sendNewOrderEmail();
			$order->setIsCustomerNotified(true);
			$order->save();
		}
		if (!empty($GLOBALS['UniMagento_NoStatusChangeOnFailedPayment'])) {
			$code = strtolower($this->uniModulName) . 'binder';
			$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderToPayInfo->shopPairingInfo);
			if (!empty($redirectAction->orderReplyStatus->successHtml)) {
				$comment = $redirectAction->orderReplyStatus->successHtml;
			} else {
				$comment = $this->__("If you were unable to make the payment, click on: <a href='%s'>RETRY PAYMENT</a>", $retryUrl);
			}
			$order->sendOrderUpdateEmail(true, $comment);
			$order->save();
		}

		if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
			$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
			$this->processReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
		}
		if ($redirectAction->redirectUrl != null) {
			Mage::app()->getResponse()->setRedirect($redirectAction->redirectUrl);
		} else if ($redirectAction->redirectForm != null) {
			$this->uniModul->webTools->formRedirect($redirectAction->redirectForm);
		}
		UniErr::end();
	}

	/**
	 * @return void
	 */
	public function replyAction() {
		UniErr::begin();
		$this->initUniModul();
		$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->getLanguage());
		$this->processReplyStatus($orderReplyStatus, true);
		UniErr::end();
	}

	 /**
	  * @return void
	  */
	 public function retryAction() {
		UniErr::begin();
		$this->redirectAction();
		UniErr::end();
	}

	/**
	 * @return void
	 */
	public function notifyAction() {
		UniErr::begin();
		$this->initUniModul();
		$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
		$this->processReplyStatus($orderReplyStatus, false);
		UniErr::end();
	}

	/**
	 * @return void
	 */
	 public function callbackAction() {
		UniErr::begin();
		$this->initUniModul();
		$this->uniModul->processCallbackRequest($_GET['_callbackName'], Http::queryParams());
		UniErr::end();
	}

	/**
	 * @param \PhpStan_Mage_Sales_Model_Order $order
	 * @param string $subMethod
	 *
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo($order, $subMethod) {
		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		$order_id = $order->getRealOrderId();
		$billing  = $order->getBillingAddress();
		if ($billing === false) {
			$billing = checkFalseOrNull($order->getShippingAddress());
		}
		/** @var \PhpStan_Mage_Sales_Model_Order_Address $billing*/

		$orderToPayInfo->shopPairingInfo = $order_id;
		$orderToPayInfo->shopOrderNumber = $order_id;

		$orderToPayInfo->amount = (float) $order->getGrandTotal();
		$orderToPayInfo->currency = $order->getOrderCurrencyCode();
		$orderToPayInfo->uniAdapterData = array('order_total'=>(float) $order->getBaseGrandTotal());

		$store = Mage::app()->getStore();
		$description = $store === null ? "" : $store->getFrontendName();
		$products = array();
		$cartItems = [];
		foreach ($order->getAllVisibleItems() as $item) {
			$products[] = $item->getName();

			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $item->getName();
			$cartItem->quantity = (float) $item->getQtyToShip();
			$cartItem->unitPrice = (float) $item->getPriceInclTax();
			$cartItem->taxRate = round($item->getTaxPercent());
			$cartItems[] = $cartItem;
		}
		$orderToPayInfo->description = $description;

		$cartItem = new UniCartItem();
		$cartItem->type = UniCartItemType::delivery;
		$cartItem->name = 'shipping / doprava';
		$cartItem->quantity = 1;
		$cartItem->unitPrice = (float) $order->getShippingInclTax();
		$cartItem->taxRate = $order->getShippingAmount() == 0 ? 0 : round(100 * $order->getShippingTaxAmount() / $order->getShippingAmount());
		$cartItems[] = $cartItem;

		$discountAmount = $order->getDiscountAmount();
		if ($discountAmount != 0) {
			// secteni pres sazby
			$rozdeleni = array_reduce($cartItems, function($a, $ci) {
				$a[(string)$ci->taxRate] = $ci->unitPrice * $ci->quantity + (isset($a[$ci->taxRate]) ? $a[$ci->taxRate] :0);
				return $a;
			}, array());


			// rovnomerne rozpocteni slevy na pasma
			$totalBeforeDiscount = $orderToPayInfo->amount - $discountAmount;
			foreach($rozdeleni as $tax=>$amt) {
				$cartItem = new UniCartItem();
				$cartItem->type = UniCartItemType::discount;
				$cartItem->name = 'discount / sleva';
				$cartItem->quantity = 1;
				$cartItem->unitPrice = $discountAmount * $amt / $totalBeforeDiscount;
				$cartItem->taxRate = $tax;
				$cartItems[] = $cartItem;
			}


		}


		$orderToPayInfo->cartItems = $cartItems;

		$customerData = new CustomerData();
		try {
		$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $order->getCustomerFirstname());
		$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $order->getCustomerLastname());
		$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $order->getCustomerEmail());
		} catch (Exception $ex) {
			$customerData->first_name = ($order->getCustomerFirstname());
			$customerData->last_name = ($order->getCustomerLastname());
			$customerData->email = ($order->getCustomerEmail());

		}
		$street = $billing->getStreet(-1);
		if (is_array($street)) {
			$customerData->street = join("; ", $street);
		} else {
			$customerData->street = $street;
		}
		$customerData->city = $billing->getCity();
		$customerData->post_code = $billing->getPostcode();
		$customerData->country = $billing->getCountryModel()->getIso2Code();
		$customerData->phone = $billing->getTelephone();
		$customerData->identifier = $billing->getCustomer_id();  // pro guesta je to null
		$orderToPayInfo->customerData = $customerData;

		$code = strtolower($this->uniModulName) . 'binder';
		$orderToPayInfo->notifyUrl = Mage::getUrl($code . '/' . $code . '/notify');
		$orderToPayInfo->replyUrl = Mage::getUrl($code . '/' . $code . '/reply');
		if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
			$orderToPayInfo->uniModulDirUrl = Mage::getBaseUrl(\Mage_Core_Model_Store::URL_TYPE_WEB) . DS . "app/code/local/Platiti/".ucfirst(strtolower($this->uniModulName))."binder/UniModul/";
		} else {
			$orderToPayInfo->uniModulDirUrl = Mage::getBaseUrl(\Mage_Core_Model_Store::URL_TYPE_WEB) . DS . "UniModul/";
		}

		$orderToPayInfo->language = $this->getLanguage();

		$model = new UniAdapterModel();
		$orderToPayInfo->currencyRates = $model->getCurrencyRates();

		//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty

		return $orderToPayInfo;
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 *
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
		$this->processReplyStatus($orderReplyStatus, false);
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param bool $frontend_redir
	 *
	 * @return void
	 */
	function processReplyStatus($orderReplyStatus, $frontend_redir) {


		$cfg = $this->getConfigDataUni($this->uniModul->name, (string) $this->uniModul->subMethod)->configData;

		BeginSynchronized();

		/** @var \PhpStan_Mage_Sales_Model_Order */
		$order = checkAndGetSingleton('sales/order');
		$order->loadByIncrementId($orderReplyStatus->shopPairingInfo);
		if (!$order->getId()) {
			// TODO  nacteni se nepovedlo...
			throw new Exception("Failed to load order!");
		}


		if ($orderReplyStatus->orderStatus == OrderStatus::$failedRetriable || $orderReplyStatus->orderStatus == OrderStatus::$failedFinal) {
			if (!$order->getEmailSent()) {
				$order->sendNewOrderEmail();
				$order->setIsCustomerNotified(true);
				$order->save();
			}
			if (!empty($GLOBALS['UniMagento_NoStatusChangeOnFailedPayment'])) {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => no effect when global UniMagento_NoStatusChangeOnFailedPayment set, shop order number ".$order->getRealOrderId() . ', sending email for retry payment');
				if (empty($orderReplyStatus->uniAdapterData['retryEmailSent'])) {
					$code = strtolower($this->uniModulName) . 'binder';
					$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderReplyStatus->shopPairingInfo);
					$comment = $this->__("Order payment failed, please try again: <a href='%s'>RETRY PAYMENT</a>", $retryUrl);
					$order->sendOrderUpdateEmail(true, $comment);
					$order->save();
					$transactionRecord = $this->uniModul->getOrderTransactionRecordFromDbUnique($orderReplyStatus->gwOrderNumber);
					if (is_null($transactionRecord)) throw new Exception('Missing transactionRecord');
					$this->uniModul->updateUniAdapterDataInDb($transactionRecord->transactionPK, array('retryEmailSent' => true) + $transactionRecord->uniAdapterData);
					$this->uniModul->logger->writeLog("ADAPTER: DEBUG: retry payment email sent");
				} else {
					//$this->uniModul->logger->writeLog("ADAPTER: DEBUG: retry payment email not sent");
				}
			} else if (!in_array($order->getState(), array(\Mage_Sales_Model_Order::STATE_NEW, \Mage_Sales_Model_Order::STATE_CANCELED, \Mage_Sales_Model_Order::STATE_PENDING_PAYMENT))) {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());
			} else {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => Canceling order ".$order->getRealOrderId());

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}
				$order->registerCancellation($comment);
				$order->setStatus($cfg['orderStatusFailed']);
				$order->sendOrderUpdateEmail(true);
				$order->save();
			}


		} else if ($orderReplyStatus->orderStatus == OrderStatus::$successful ) {

			if (!in_array($order->getState(), array(\Mage_Sales_Model_Order::STATE_NEW, \Mage_Sales_Model_Order::STATE_CANCELED, \Mage_Sales_Model_Order::STATE_PENDING_PAYMENT))) {

				$this->uniModul->logger->writeLog("ADAPTER: successful => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());

			} else {

				$this->uniModul->logger->writeLog("ADAPTER: successful => registerCaptureNotification for shop order number ".$order->getRealOrderId());

				$comment = "";
				if ($orderReplyStatus->forexNote !== null) {
					$comment = $this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				/** @var \PhpStan_Mage_Sales_Model_Order_Payment*/
				$payment = checkFalseOrNull($order->getPayment());
				if (is_null($orderReplyStatus->gwOrderNumber)) throw new Exception('gwOrderNumber missing');
				$payment->setTransactionId($orderReplyStatus->gwOrderNumber)
					->setIsTransactionClosed(false)
					->registerCaptureNotification($orderReplyStatus->uniAdapterData['order_total'], !empty($GLOBALS['UniMagento_skipFraudDetection']));
					if ($comment != "") {
						$payment->setPreparedMessage($comment);
					}
				$order->setStatus($cfg['orderStatusSuccessfull']);
				$order->save();

				$invoice = $payment->getCreatedInvoice();
				if ($invoice && !$order->getEmailSent()) {
					$order->sendNewOrderEmail();
					$order->setIsCustomerNotified(true);
				}
				$order->sendOrderUpdateEmail(true);
				$order->save();
			}


		} else if ($orderReplyStatus->orderStatus == OrderStatus::$pending ) {

			if (!in_array($order->getState(), array(\Mage_Sales_Model_Order::STATE_NEW))) {

				$this->uniModul->logger->writeLog("ADAPTER: pending => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());

			} else {

				$this->uniModul->logger->writeLog("ADAPTER: pending => setState STATE_PENDING_PAYMENT for shop order number ".$order->getRealOrderId());

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				/** @var \PhpStan_Mage_Sales_Model_Order_Payment*/
				$payment =  checkFalseOrNull($order->getPayment());
				if (is_null($orderReplyStatus->gwOrderNumber)) throw new Exception('gwOrderNumber missing');
				$payment->setTransactionId($orderReplyStatus->gwOrderNumber)
					->setIsTransactionClosed(false);
				$order->setState(\Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, $cfg['orderStatusPending'], $comment, true);
				if (!$order->getEmailSent()) {
					$order->sendNewOrderEmail();
					$order->setIsCustomerNotified(true);
				}
				$order->save();
			}

		} else if ($orderReplyStatus->orderStatus == OrderStatus::$invalidReply) {
			$this->uniModul->logger->writeLog("ADAPTER: invalidReply => no effect");
		} else {
			$this->uniModul->logger->writeLog("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
			user_error("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
		}

		EndSynchronized();

		if ($frontend_redir) {
			if ($orderReplyStatus->orderStatus == OrderStatus::$pending || $orderReplyStatus->orderStatus == OrderStatus::$successful) {

				// zrusime kosik, pokdu byl predtim obnoven viz nize
				/** @var \Mage_Checkout_Model_Session */
				$session = checkAndGetSingleton('checkout/session');
				$lastQuoteId = $session->getLastQuoteId();
				/** @var \Mage_Sales_Model_Quote */
				$quoteModel = checkAndGetModel('sales/quote');
				$quote = $quoteModel ->load($lastQuoteId);
				$quote->setIsActive(0)->save();

				/** @var \Mage_Core_Model_Session */
				$core_session = checkAndGetSingleton('core/session');
				$core_session->setData('UniModulSuccessHtmlUniModul', $this->uniModulName);
				$core_session->setData('UniModulSuccessHtml', $orderReplyStatus->successHtml);
				$this->_redirect('checkout/onepage/success', array('_secure'=>true));
			} else {

				// obnovime kosik
				/** @var \Mage_Checkout_Model_Session */
				$session = checkAndGetSingleton('checkout/session');
				$lastQuoteId = $session->getLastQuoteId();
				/** @var \Mage_Sales_Model_Quote */
				$quoteModel = checkAndGetModel('sales/quote');
				$quote = $quoteModel ->load($lastQuoteId);
				$quote->setIsActive(1)->save();

				$this->loadLayout();
				$rootBlock = checkFalseOrNull($this->getLayout()->getBlock('root'));
				$rootBlock->setTemplate('page/1column.phtml');
				/** @var \Mage_Checkout_Block_Onepage_Failure */
				$block = checkFalseOrNull($this->getLayout()->createBlock(
					'Mage_Checkout_Block_Onepage_Failure',
					'newpage',
					array('template' => 'uniadapter_layout/failure.phtml')
				));

				$code = strtolower($this->uniModulName) . 'binder';
				$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderReplyStatus->shopPairingInfo);
				$block->assign(array('retryUrl' => $retryUrl, 'errorMessage' => $orderReplyStatus->resultText, "__" => array($this, "__")));
				$contentBlock = checkFalseOrNull($this->getLayout()->getBlock('content'));
				$contentBlock->append($block);
				UniErr::begin(E_UNIERR_DEFAULT & ~E_WARNING); // v renderLayoutu zachytavme jinak chyby ve @vsprintf()
				$this->renderLayout();
				UniErr::end();
			}
		}
	}


	/**
	 * @return string
	 */
	protected function getLanguage() {
		return substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2);
	}
}
