<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento;

if (!defined('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath')) define('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath', \Mage::getBaseDir() . DS . 'var/UniModul');

require_once("UniModul.php");

use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\ConfigField;
use Platiti\GPWebPayGpeNs\UniModul\ConfigFieldType;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;
use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;

use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\create_initialize_object;

use Mage;
use Exception;

class UniAdapterModel extends \Mage_Payment_Model_Method_Abstract
{
	/**
	 * @var string
	 */
	protected $uniModulName = null; //predefinuje binder
	/**
	 * @var UniModul
	 */
	protected $uniModul = null;
	/**
	 * @var string
	 */
	protected $subMethod = '';


	/**
	 * @var string
	 */
	protected $_code = '';
	//protected $_shouldInactivateQuote = false;   //nefunguje pro zachovani kosiku
	/**
	 * @var bool
	 */
	protected $_isGateway               = true;
	/**
	 * @var bool
	 */
	protected $_isInitializeNeeded      = true;
	/**
	 * @var string
	 */
	protected $_formBlockType = 'platiti/form';
	/**
	 * @var bool
	 */
	protected $_canCapture              = true;

	/**
	 * pro získani konfigurace pro controller, pro callback je totiz nutne aby byl adapter cely inicializovany
	 *
	 * @param ?string $uniModulName
	 * @param ?string $subMethod
	 *
	 * @return mixed
	 */
	public function getConfigDataUniExternal($uniModulName = null, $subMethod = null) {
		UniErr::begin();
		if ($uniModulName != null) $this->uniModulName = $uniModulName;
		else throw new Exception("No unimodul name!");
		if ($subMethod != null) $this->subMethod = $subMethod;
		return UniErr::end($this->initUniModul());
	}

	/**
	 * @param string $paymentAction
	 * @param \PhpStan_State_Object $stateObject
	 *
	 * @return $this
	 */
	public function initialize($paymentAction, $stateObject)
	{
		UniErr::begin();
		$state = \Mage_Sales_Model_Order::STATE_NEW;
		$stateObject->setState($state);
		/** @var \Mage_Sales_Model_Order_Config */
		$order_config_model = checkAndGetSingleton('sales/order_config');
		$stateObject->setStatus($order_config_model->getStateDefaultStatus($state));
		$stateObject->setIsNotified(false);
		return UniErr::end($this);
	}

	// jinak se nevola initialize
	public function getConfigPaymentAction() {
		UniErr::begin();
		$paymentAction = $this->getConfigData('payment_action');
		return UniErr::end(empty($paymentAction) ? true : $paymentAction);
	}

	/**
	 * @return void|ConfigSetting
	 */
	protected function initUniModul() {
		if ($this->uniModul == null) {
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($this->uniModulName, $this->getLanguage(), $this->subMethod, 'Magento');
			$configSetting = $this->getConfigDataUni($configInfo, $this->uniModulName, $this->subMethod);
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
			$baseDir = Mage::getBaseDir();
			$versionPath = $baseDir . DS . "var/UniModul/platiti_magento_version.txt";
			$unimodulDir = null;
			if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
				$unimodulDir = $baseDir . DS . "app/code/local/Platiti/".ucfirst(strtolower($this->uniModulName))."binder/UniModul/";
			} else {
				$unimodulDir = $baseDir . DS . "UniModul/";
			}

			if (!file_exists($unimodulDir.'.htaccess') || file_get_contents($versionPath) !== $this->uniModul->versionStr) {
				if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
					$this->uniModul->moduleInstallVarDir();
				}
				$this->uniModul->moduleInstallDatabase();
				file_put_contents(
					$unimodulDir . '.htaccess',
					"# Deny access to everything by default\n" .
					"Deny from all\n\n" .
					"# Allow access to specific image file types\n" .
					"<FilesMatch \"\\.(jpeg|jpg|png|gif|webp)$\">\n" .
					"	Allow from all\n" .
					"</FilesMatch>\n" .
					"<FilesMatch \"testGW.php$\">\n" .
					"	Allow from all\n" .
					"</FilesMatch>\n"
				);
				file_put_contents(
					$versionPath,
					$this->uniModul->versionStr
				);
			}
			return $configSetting;
		}
	}


	/**
	 * @return string
	 */
	public function getOrderPlaceRedirectUrl() {
		UniErr::begin();
		//mage::log('Called custom ' . __METHOD__);
		/** @var \PhpStan_Mage_Checkout_Model_Session */
		$session = checkAndGetSingleton('checkout/session');
		$session->setUniAdapterSubmethod($this->subMethod);
		return UniErr::end(Mage::getUrl(strtolower($this->uniModulName) . 'binder/' . strtolower($this->uniModulName) . 'binder/redirect', array('_secure' => true)));
	}

	public function isAvailable($quote = null) {
		UniErr::begin();
		if (is_null($this->uniModulName)) throw new Exception("No unimodul name!"); // @phpstan-ignore-line better than checking if is null everywhere
		if (!parent::isAvailable($quote)) {
			return UniErr::end(false);
		}
		if ($quote == null) {  // nevim proc se to muze volat s null, ale budeme optimisti
			return UniErr::end(true);
		}

		$orderToPayInfo = $this->getOrderToPayInfo($quote, $this->subMethod);

		$this->initUniModul();
		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);

		return UniErr::end($prePayGWInfo->isPossible);
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param string $uniModulName
	 * @param string $subMethod
	 *
	 * @return ConfigSetting
	 */
	protected function getConfigDataUni($configInfo, $uniModulName, $subMethod) {

		// doplnime failed stav, i kdyz ho modul nevyzaduje, protoze Magento vzdy vytvori order
		if (array_filter($configInfo->configFields, function (ConfigField $e) { return $e->name == 'orderStatusFailed'; }) === array()) {
			$configField = new ConfigField();
			$configField->name = 'orderStatusFailed';
			$configField->type = ConfigFieldType::$orderStatus;
			$configInfo->configFields[] = $configField;
		}

		$configData = array();
		$path = "payment/" . strtolower($uniModulName) . "binder/";
		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = Mage::getStoreConfig($path . $configField->name);
		}

		$uniModulConfig = new UniModulConfig();

		$connConfig = checkFalseOrNull(checkFalseOrNull(Mage::getConfig())->getResourceConnectionConfig('core_write'));

		$uniModulConfig->mysql_server = (string)$connConfig->host;
		$uniModulConfig->mysql_dbname = (string)$connConfig->dbname;
		$uniModulConfig->mysql_login = (string)$connConfig->username;
		$uniModulConfig->mysql_password = (string)$connConfig->password;

		$dbtype = $connConfig->type;
		if ($dbtype != 'mysql' && $dbtype != 'mysqli' && $dbtype != 'pdo_mysql') user_error('Eshop pouziva jinou nez mysql databazi');

		/** @var \Mage_Core_Model_Url */
		$url_model = checkAndGetModel('core/url');
		$baseUrl = $url_model->getStore()->getBaseUrl(\Mage_Core_Model_Store::URL_TYPE_WEB);
		if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
			$uniModulConfig->uniModulDirUrl = $baseUrl . "app/code/local/Platiti/". ucfirst(strtolower($uniModulName))."binder/UniModul/";
		} else {
			$uniModulConfig->uniModulDirUrl = $baseUrl . "UniModul/";
		}

		$uniModulConfig->legacyUniModulVarPaths = [Mage::getBaseDir() . '/UniModul'];
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');

		$uniModulConfig->adapterName = 'Magento';
		$cfgs = create_initialize_object(ConfigSetting::class, array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));

		return $cfgs;

	}

	/**
	 * @param \Mage_Sales_Model_Quote $quote
	 * @param string $subMethod
	 *
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo($quote, $subMethod) {
		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		//$order_id = $quote->getRealOrderId();
		$billing  = checkFalseOrNull($quote->getBillingAddress());

		//$orderToPayInfo->shopPairingInfo = $order_id;
		//$orderToPayInfo->shopOrderNumber = $order_id;

		$orderToPayInfo->amount = (float) $quote->getGrandTotal();
		$orderToPayInfo->currency = $quote->getQuoteCurrencyCode();

		$customerData = new CustomerData();
		$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $quote->getCustomerFirstname());
		$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $quote->getCustomerLastname());
		$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $quote->getCustomerEmail());
		$street = $billing->getStreet(-1);
		if (is_array($street)) {
			$customerData->street = join("; ", $street);
		} else {
			$customerData->street = $street;
		}    	$customerData->city = $billing->getCity();
		$customerData->post_code = $billing->getPostcode();
		$customerData->country = $billing->getCountryModel()->getIso3Code();
		$customerData->phone = $billing->getTelephone();
		$orderToPayInfo->customerData = $customerData;

		// toto orderToPayInfo neni urceno k presmerovani na branu
		//$code = strtolower($this->uniModulName) . 'binder';
		//$orderToPayInfo->notifyUrl = Mage::getUrl($code . '/' . $code . '/notify', array('transaction_id_XX' => $order_id));
		//$orderToPayInfo->replyUrl = Mage::getUrl($code . '/' . $code . '/reply', array('transaction_id_XX' => $order_id));

		$orderToPayInfo->language = $this->getLanguage();

		$orderToPayInfo->currencyRates = $this->getCurrencyRates();

		//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty

		return $orderToPayInfo;
	}

	/**
	 * @return array<string, float>
	 */
	public function getCurrencyRates() {
		$currencyRates = array();
		$store = checkFalseOrNull(Mage::app()->getStore(null));
		$cur = $store->getBaseCurrency();
		foreach($cur->getConfigAllowCurrencies() as $c) {
			$currencyRates[(string) $c] = (float) $cur->getAnyRate($c);
		}
		return $currencyRates;
	}

	/**
	 * @param string $callbackName
	 * @param array<mixed, mixed> $arguments
	 *
	 * @return string
	 */
	public function getCallbackUrl($callbackName, $arguments) {
		$code = strtolower($this->uniModulName) . 'binder';
		return Mage::getUrl($code . '/' . $code . '/callback')."?_unimodul=".$this->uniModul->name."&_callbackName=".urlencode($callbackName)."&".http_build_query($arguments);;
	}

	 /**
	  * @return string
	  */
	 function getMiniLogo() {
		return $this->uniModul->getMiniLogoSpan();
	}

	/**
	 * @return string
	 */
	function getImage() {
		return $this->uniModul->getModulSubMethodLogoImage2($this->uniModul->subMethod);
	}



	/**
	 * @return string
	 */
	public function getInstallmentHtml() {
		UniErr::begin();
		$currentProduct = Mage::registry('current_product');
		if (!is_null($currentProduct) && parent::isAvailable(null)) {
			$store = checkFalseOrNull(Mage::app()->getStore(null));
			if ($currentProduct->getTypeId() == \Mage_Catalog_Model_Product_Type::TYPE_BUNDLE) {
				$priceModel = $currentProduct->getPriceModel();
				list($_minimalPriceInclTax, $_maximalPriceInclTax) = $priceModel->getTotalPrices($currentProduct, null, true, false);
				$price = $_maximalPriceInclTax;
			} else {
				$price = $currentProduct->getFinalPrice();
			}

			$product_price = $store->getBaseCurrency()->convert($price, $store->getCurrentCurrency());
			$current_currency = $store->getCurrentCurrencyCode();

			/** @var \Mage_Core_Model_Url */
			$url_model = checkAndGetModel('core/url');
			$baseUrl = $url_model->getStore()->getBaseUrl(\Mage_Core_Model_Store::URL_TYPE_WEB);
			$this->initUniModul();
			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml($baseUrl, $current_currency, $product_price);

			return UniErr::end((string) $embedHtml);
		} else {
			return UniErr::end('');
		}
	}

	/**
	 * @return string
	 */
	public function getSuccessHtml() {
		UniErr::begin();
		/** @var \Mage_Core_Model_Session */
		$session = checkAndGetSingleton('core/session');
		if ($session->getData('UniModulSuccessHtmlUniModul') == $this->uniModulName && !empty($session->getData('UniModulSuccessHtml'))) {
			$html = "<style>.uniModulSuccessHtml td { padding-right:1ex; }</style><div class='main container'><div class='uniModulSuccessHtml'><p>&nbsp;<p/>".$session->getData('UniModulSuccessHtml')."</div></div>";
			$session->setData('UniModulSuccessHtml', null);
			return UniErr::end($html);
		} else {
			return UniErr::end('');
		}
	}

	/**
	 * @return string
	 */
	protected function getLanguage() {
		return substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2);
	}
}


/**
 * @param string $modelClass
 * @param mixed[] $arguments
 * @return \Mage_Core_Model_Abstract
 */
function checkAndGetSingleton($modelClass, $arguments = array()) {
	$singleton = Mage::getSingleton($modelClass, $arguments);
	if ($singleton === false) {
		throw new Exception("getSingleton function returned false");
	}
	return $singleton;
}

/**
 * @param string $modelClass
 * @param mixed[]|string|object $arguments
 * @return \Mage_Core_Model_Abstract
 */
function checkAndGetModel($modelClass, $arguments = array()) {
	$model = Mage::getModel($modelClass, $arguments);
	if ($model === false) {
		throw new Exception("getModel function returned false");
	}
	return $model;
}

/**
 * @template T
 * @param T|false|null $value
 * @return T
 * @throws \InvalidArgumentException
 */
function checkFalseOrNull($value) {
	if ($value === false || $value === null) {
		throw new \InvalidArgumentException("The input value must not be false.");
	}
	return $value;
}